@@ -6,7 +6,7 @@
 # Author: jianglin
 # Email: xiyang0807@gmail.com
 # Created: 2018-02-26 11:41:22 (CST)
-# Last Update: Tuesday 2018-02-27 09:53:40 (CST)
+# Last Update: Wednesday 2018-02-28 15:09:40 (CST)
 #          By:
 # Description:
 # ********************************************************************************
@@ -14,26 +14,21 @@ from collections import OrderedDict
 
 from .regex import Regex
 
-html_escape_table = {
-    # "&": "&amp;",
-    # '"': "&quot;",
-    # "'": "&apos;",
-    ">": "&gt;",
-    "<": "&lt;",
-}
 
-
-def html_escape(text):
-    """Produce entities within text."""
-    return "".join(html_escape_table.get(c, c) for c in text)
+def html_escape(text, quote=False):
+    text = text.replace('<', '&lt;')
+    text = text.replace('>', '&gt;')
+    if quote:
+        text = text.replace('"', '&quot;')
+        text = text.replace("'", '&#39;')
+    return text
 
 
 class InlineElement(object):
     label = '{text}'
 
-    def __init__(self, text, regex):
+    def __init__(self, text):
         self.text = text
-        self.regex = regex
         self.children = []
 
     def to_html(self):
@@ -50,12 +45,16 @@ class InlineElement(object):
     def __str__(self):
         return '{}({})'.format(self.__class__.__name__, self.text.strip())
 
+    def __repr__(self):
+        return self.__str__()
+
 
 class Fn(InlineElement):
     '''
     <sup><a id="fnr.1" class="footref" href="#fn.1">1</a></sup>
     '''
     label = '<sup><a id="fnr:{text}" class="footref" href="#fn.{text}">{text}</a></sup>'
+    regex = Regex.fn
 
     def to_html(self):
         def _match(match):
@@ -64,37 +63,53 @@ class Fn(InlineElement):
         return self.regex.sub(_match, self.text)
 
 
+class Comment(InlineElement):
+    regex = Regex.comment
+
+    def to_html(self):
+        return self.text
+
+
 class Underlined(InlineElement):
     label = '<span style="text-decoration:underline">{text}</span>'
+    regex = Regex.underlined
 
 
 class Bold(InlineElement):
     label = '<b>{text}</b>'
+    regex = Regex.bold
 
 
 class Italic(InlineElement):
     label = '<i>{text}</i>'
+    regex = Regex.italic
 
 
 class Code(InlineElement):
     label = '<code>{text}</code>'
+    regex = Regex.code
 
 
 class Delete(InlineElement):
     label = '<del>{text}</del>'
+    regex = Regex.delete
 
 
 class Verbatim(InlineElement):
     label = '<code>{text}</code>'
+    regex = Regex.verbatim
 
 
 class NewLine(InlineElement):
+    regex = Regex.newline
+
     def to_html(self):
         return self.regex.sub('<br/>', self.text)
 
 
 class Image(InlineElement):
     label = '<img src="{text}"/>'
+    regex = Regex.image
 
     def to_html(self):
         def _match(match):
@@ -105,19 +120,22 @@ class Image(InlineElement):
 
 class Link(InlineElement):
     label = '<a href="{href}">{text}</a>'
+    regex = Regex.link
 
     def to_html(self):
         def _match(match):
-            if not match.group('text'):
-                return Image.label.format(text=match.group('href'))
-            return self.label.format(
-                href=match.group('href'), text=match.group('text'))
+            text = match.group('text')
+            href = match.group('href')
+            if not text:
+                return Image.label.format(text=href)
+            return self.label.format(href=href, text=text)
 
         return self.regex.sub(_match, self.text)
 
 
 class OriginLink(InlineElement):
     label = '<a href="{0}">{0}</a>'
+    regex = Regex.origin_link
 
     def to_html(self):
         def _match(match):
@@ -128,67 +146,40 @@ class OriginLink(InlineElement):
 
 class Text(InlineElement):
     regex = OrderedDict([
-        ('comment', Regex.comment),
-        ('newline', Regex.newline),
-        ('italic', Regex.italic),
-        ('bold', Regex.bold),
-        ('underlined', Regex.underlined),
-        ('code', Regex.code),
-        ('delete', Regex.delete),
-        ('verbatim', Regex.verbatim),
-        ('fn', Regex.fn),
-        ('link', Regex.link),
-        ('image', Regex.image),
+        ('comment', Comment),
+        ('newline', NewLine),
+        ('italic', Italic),
+        ('bold', Bold),
+        ('underlined', Underlined),
+        ('code', Code),
+        ('delete', Delete),
+        ('verbatim', Verbatim),
+        ('fn', Fn),
+        ('link', Link),
+        ('image', Image),
     ])
 
-    def __init__(self, text, force=True):
+    def __init__(self, text, force=True, escape=False):
         self.text = text
         self.force = force
-        if self.force:
-            self.text = html_escape(self.text)
+
+        if escape:
+            self.text = html_escape(text)
+
+    def parse_comment(self, text):
+        return ""
 
     def parse(self, text):
         if not isinstance(text, str):
             text = text.to_html()
         if not self.force:
             return text
-        for parse, regex in self.regex.items():
-            if regex.search(text):
-                return getattr(self, 'parse_' + parse)(text, regex)
+        for name, element in self.regex.items():
+            if element.regex.search(text):
+                name = "parse_" + name
+                return getattr(self, name)(text) if hasattr(
+                    self, name) else self.parse(element(text))
         return text
 
-    def parse_comment(self, text, regex):
-        return text
-
-    def parse_newline(self, text, regex):
-        return self.parse(NewLine(text, regex))
-
-    def parse_italic(self, text, regex):
-        return self.parse(Italic(text, regex))
-
-    def parse_bold(self, text, regex):
-        return self.parse(Bold(text, regex))
-
-    def parse_underlined(self, text, regex):
-        return self.parse(Underlined(text, regex))
-
-    def parse_code(self, text, regex):
-        return self.parse(Code(text, regex))
-
-    def parse_delete(self, text, regex):
-        return self.parse(Delete(text, regex))
-
-    def parse_verbatim(self, text, regex):
-        return self.parse(Verbatim(text, regex))
-
-    def parse_fn(self, text, regex):
-        return self.parse(Fn(text, regex))
-
-    def parse_link(self, text, regex):
-        return self.parse(Link(text, regex))
-
-    def parse_image(self, text, regex):
-        return self.parse(Image(text, regex))
-
     def to_html(self):
         return self.parse(self.text)
